import { AxiosWrapper } from '@/utils/axios-wrapper'
import { UserStore } from '@/store/user'
import { cloneDeep, isArray, omit } from 'lodash-es'
import NProgress from 'nprogress'
import 'nprogress/nprogress.css'
const whiteList = ['/redirect', '/login', '/401', '/404']
NProgress.configure({ showSpinner: false }) // 进度条
// 获取用户信息,返回菜单
const getUserMeta = async (token) => {
  const userStore = UserStore()
  const axios = new AxiosWrapper()
  const result = await axios.$get(`/token/tokenView`, null, {
    headers: {
      token: token
    }
  })
  if (result.errcode == 0) {
    userStore.setUser(result.data)
    const resetMenusResult = resetMenus(result.data.menus)
    const storeMenus = toRaw(userStore.menus)
    if (storeMenus.length <= 0) {
      userStore.setMenus(resetMenusResult)
    }
    return { menus: result.data.menus, errcode: 0 }
  }
  return { errmsg: result.errmsg, errcode: result.errcode }
}

const resetMenus = (menus) => {
  if (!isArray(menus) || menus.length <= 0) return []
  const cMenus = cloneDeep(menus)
  const result = []
  for (const m of cMenus) {
    const mid = omit(m, ['is_use', 'order_num', 'in_admin_frame'])
    const { children } = mid
    if (children) mid.children = resetMenus(children)
    result.push(mid)
  }
  return result
}

// 注册前置守卫
export const registerBeforeRouter = async (router) => {
  router.beforeEach(async (to, from, next) => {
    NProgress.start() //开启进度条
    const token = localStorage.getItem('admin-token')
    NProgress.inc()
    if (whiteList.includes(to.path)) {
      next()
      return
    }
    if (token) {
      NProgress.inc()
      if (to.path === '/login') next()
      NProgress.inc()
      await getUserMeta(token)
      NProgress.inc()
      // 注册了直接进入
      next()
    } else {
      next('/login')
    }
  })
}

// 注册路由后置守卫

export const registerAfterRouter = async (router) => {
  router.afterEach(async (to, form) => {
    NProgress.done() //完成进度条
    if (to.path === '/login') {
      return
    }
    // 请求该页面的权限
  })
}
